const uuidv4 = require('uuid/v4');

const CPS_INFO_FIELDS = ['cookie_id', 'first_time', 'siteId', 'pid'];

const reducers = {
  cookie_id(value) {
    if (value) {
      return value;
    }

    return uuidv4();
  },

  first_time(value) {
    if (value && value instanceof Date) {
      return value.toISOString();
    }

    if (typeof value === 'string' && !isNaN(new Date(value).getTime())) {
      return value;
    }

    return (new Date()).toISOString();
  },
};

function getField(data, name) {
  const value = data[name];
  const reduce = reducers[name];
  if (reduce) {
    return reduce(value);
  }

  return value;
}

export function createCpsInfoObj(data) {
  const obj = {};
  CPS_INFO_FIELDS.forEach((name) => {
    obj[name] = getField(data, name);
  });
  return obj;
}

export function parseCpsInfoStr(str) {
  return JSON.parse(decodeURIComponent(str));
}

// may throws, in which case nothing should be working, so ..
export default function createCpsInfoCookie(data) {
  const obj = createCpsInfoObj(data);
  return `cpsInfo=${encodeURIComponent(JSON.stringify(obj))}`;
}
